# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_SET_SPACE_METADATA = Metadata(
    id="39cfd66daa80ee755b48163f61a64435ef0a12be.workbench",
    name="volume-set-space",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeSetSpacePlumbParamsDictNoTag = typing.TypedDict('_VolumeSetSpacePlumbParamsDictNoTag', {
    "axis-order": str,
    "x-spacing": float,
    "y-spacing": float,
    "z-spacing": float,
    "x-offset": float,
    "y-offset": float,
    "z-offset": float,
})
VolumeSetSpacePlumbParamsDictTagged = typing.TypedDict('VolumeSetSpacePlumbParamsDictTagged', {
    "@type": typing.Literal["plumb"],
    "axis-order": str,
    "x-spacing": float,
    "y-spacing": float,
    "z-spacing": float,
    "x-offset": float,
    "y-offset": float,
    "z-offset": float,
})
VolumeSetSpacePlumbParamsDict = _VolumeSetSpacePlumbParamsDictNoTag | VolumeSetSpacePlumbParamsDictTagged


_VolumeSetSpaceSformParamsDictNoTag = typing.TypedDict('_VolumeSetSpaceSformParamsDictNoTag', {
    "xi-spacing": float,
    "xj-spacing": float,
    "xk-spacing": float,
    "x-offset": float,
    "yi-spacing": float,
    "yj-spacing": float,
    "yk-spacing": float,
    "y-offset": float,
    "zi-spacing": float,
    "zj-spacing": float,
    "zk-spacing": float,
    "z-offset": float,
})
VolumeSetSpaceSformParamsDictTagged = typing.TypedDict('VolumeSetSpaceSformParamsDictTagged', {
    "@type": typing.Literal["sform"],
    "xi-spacing": float,
    "xj-spacing": float,
    "xk-spacing": float,
    "x-offset": float,
    "yi-spacing": float,
    "yj-spacing": float,
    "yk-spacing": float,
    "y-offset": float,
    "zi-spacing": float,
    "zj-spacing": float,
    "zk-spacing": float,
    "z-offset": float,
})
VolumeSetSpaceSformParamsDict = _VolumeSetSpaceSformParamsDictNoTag | VolumeSetSpaceSformParamsDictTagged


_VolumeSetSpaceFileParamsDictNoTag = typing.TypedDict('_VolumeSetSpaceFileParamsDictNoTag', {
    "volume-ref": str,
    "ignore-dims": bool,
})
VolumeSetSpaceFileParamsDictTagged = typing.TypedDict('VolumeSetSpaceFileParamsDictTagged', {
    "@type": typing.Literal["file"],
    "volume-ref": str,
    "ignore-dims": bool,
})
VolumeSetSpaceFileParamsDict = _VolumeSetSpaceFileParamsDictNoTag | VolumeSetSpaceFileParamsDictTagged


_VolumeSetSpaceParamsDictNoTag = typing.TypedDict('_VolumeSetSpaceParamsDictNoTag', {
    "volume-in": InputPathType,
    "volume-out": str,
    "plumb": typing.NotRequired[VolumeSetSpacePlumbParamsDict | None],
    "sform": typing.NotRequired[VolumeSetSpaceSformParamsDict | None],
    "file": typing.NotRequired[VolumeSetSpaceFileParamsDict | None],
})
VolumeSetSpaceParamsDictTagged = typing.TypedDict('VolumeSetSpaceParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-set-space"],
    "volume-in": InputPathType,
    "volume-out": str,
    "plumb": typing.NotRequired[VolumeSetSpacePlumbParamsDict | None],
    "sform": typing.NotRequired[VolumeSetSpaceSformParamsDict | None],
    "file": typing.NotRequired[VolumeSetSpaceFileParamsDict | None],
})
VolumeSetSpaceParamsDict = _VolumeSetSpaceParamsDictNoTag | VolumeSetSpaceParamsDictTagged


def volume_set_space_plumb(
    axis_order: str,
    x_spacing: float,
    y_spacing: float,
    z_spacing: float,
    x_offset: float,
    y_offset: float,
    z_offset: float,
) -> VolumeSetSpacePlumbParamsDictTagged:
    """
    Build parameters.
    
    Args:
        axis_order: a string like 'XYZ' that specifies which index is along\
            which spatial dimension.
        x_spacing: change in x-coordinate from incrementing the relevant index.
        y_spacing: change in y-coordinate from incrementing the relevant index.
        z_spacing: change in z-coordinate from incrementing the relevant index.
        x_offset: the x-coordinate of the first voxel.
        y_offset: the y-coordinate of the first voxel.
        z_offset: the z-coordinate of the first voxel.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "plumb",
        "axis-order": axis_order,
        "x-spacing": x_spacing,
        "y-spacing": y_spacing,
        "z-spacing": z_spacing,
        "x-offset": x_offset,
        "y-offset": y_offset,
        "z-offset": z_offset,
    }
    return params


def volume_set_space_plumb_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeSetSpacePlumbParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("axis-order", None) is None:
        raise StyxValidationError("`axis-order` must not be None")
    if not isinstance(params["axis-order"], str):
        raise StyxValidationError(f'`axis-order` has the wrong type: Received `{type(params.get("axis-order", None))}` expected `str`')
    if params.get("x-spacing", None) is None:
        raise StyxValidationError("`x-spacing` must not be None")
    if not isinstance(params["x-spacing"], (float, int)):
        raise StyxValidationError(f'`x-spacing` has the wrong type: Received `{type(params.get("x-spacing", None))}` expected `float`')
    if params.get("y-spacing", None) is None:
        raise StyxValidationError("`y-spacing` must not be None")
    if not isinstance(params["y-spacing"], (float, int)):
        raise StyxValidationError(f'`y-spacing` has the wrong type: Received `{type(params.get("y-spacing", None))}` expected `float`')
    if params.get("z-spacing", None) is None:
        raise StyxValidationError("`z-spacing` must not be None")
    if not isinstance(params["z-spacing"], (float, int)):
        raise StyxValidationError(f'`z-spacing` has the wrong type: Received `{type(params.get("z-spacing", None))}` expected `float`')
    if params.get("x-offset", None) is None:
        raise StyxValidationError("`x-offset` must not be None")
    if not isinstance(params["x-offset"], (float, int)):
        raise StyxValidationError(f'`x-offset` has the wrong type: Received `{type(params.get("x-offset", None))}` expected `float`')
    if params.get("y-offset", None) is None:
        raise StyxValidationError("`y-offset` must not be None")
    if not isinstance(params["y-offset"], (float, int)):
        raise StyxValidationError(f'`y-offset` has the wrong type: Received `{type(params.get("y-offset", None))}` expected `float`')
    if params.get("z-offset", None) is None:
        raise StyxValidationError("`z-offset` must not be None")
    if not isinstance(params["z-offset"], (float, int)):
        raise StyxValidationError(f'`z-offset` has the wrong type: Received `{type(params.get("z-offset", None))}` expected `float`')


def volume_set_space_plumb_cargs(
    params: VolumeSetSpacePlumbParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-plumb",
        params.get("axis-order", None),
        str(params.get("x-spacing", None)),
        str(params.get("y-spacing", None)),
        str(params.get("z-spacing", None)),
        str(params.get("x-offset", None)),
        str(params.get("y-offset", None)),
        str(params.get("z-offset", None))
    ])
    return cargs


def volume_set_space_sform(
    xi_spacing: float,
    xj_spacing: float,
    xk_spacing: float,
    x_offset: float,
    yi_spacing: float,
    yj_spacing: float,
    yk_spacing: float,
    y_offset: float,
    zi_spacing: float,
    zj_spacing: float,
    zk_spacing: float,
    z_offset: float,
) -> VolumeSetSpaceSformParamsDictTagged:
    """
    Build parameters.
    
    Args:
        xi_spacing: increase in x coordinate from incrementing the i index.
        xj_spacing: increase in x coordinate from incrementing the j index.
        xk_spacing: increase in x coordinate from incrementing the k index.
        x_offset: x coordinate of first voxel.
        yi_spacing: increase in y coordinate from incrementing the i index.
        yj_spacing: increase in y coordinate from incrementing the j index.
        yk_spacing: increase in y coordinate from incrementing the k index.
        y_offset: y coordinate of first voxel.
        zi_spacing: increase in z coordinate from incrementing the i index.
        zj_spacing: increase in z coordinate from incrementing the j index.
        zk_spacing: increase in z coordinate from incrementing the k index.
        z_offset: z coordinate of first voxel.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "sform",
        "xi-spacing": xi_spacing,
        "xj-spacing": xj_spacing,
        "xk-spacing": xk_spacing,
        "x-offset": x_offset,
        "yi-spacing": yi_spacing,
        "yj-spacing": yj_spacing,
        "yk-spacing": yk_spacing,
        "y-offset": y_offset,
        "zi-spacing": zi_spacing,
        "zj-spacing": zj_spacing,
        "zk-spacing": zk_spacing,
        "z-offset": z_offset,
    }
    return params


def volume_set_space_sform_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeSetSpaceSformParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("xi-spacing", None) is None:
        raise StyxValidationError("`xi-spacing` must not be None")
    if not isinstance(params["xi-spacing"], (float, int)):
        raise StyxValidationError(f'`xi-spacing` has the wrong type: Received `{type(params.get("xi-spacing", None))}` expected `float`')
    if params.get("xj-spacing", None) is None:
        raise StyxValidationError("`xj-spacing` must not be None")
    if not isinstance(params["xj-spacing"], (float, int)):
        raise StyxValidationError(f'`xj-spacing` has the wrong type: Received `{type(params.get("xj-spacing", None))}` expected `float`')
    if params.get("xk-spacing", None) is None:
        raise StyxValidationError("`xk-spacing` must not be None")
    if not isinstance(params["xk-spacing"], (float, int)):
        raise StyxValidationError(f'`xk-spacing` has the wrong type: Received `{type(params.get("xk-spacing", None))}` expected `float`')
    if params.get("x-offset", None) is None:
        raise StyxValidationError("`x-offset` must not be None")
    if not isinstance(params["x-offset"], (float, int)):
        raise StyxValidationError(f'`x-offset` has the wrong type: Received `{type(params.get("x-offset", None))}` expected `float`')
    if params.get("yi-spacing", None) is None:
        raise StyxValidationError("`yi-spacing` must not be None")
    if not isinstance(params["yi-spacing"], (float, int)):
        raise StyxValidationError(f'`yi-spacing` has the wrong type: Received `{type(params.get("yi-spacing", None))}` expected `float`')
    if params.get("yj-spacing", None) is None:
        raise StyxValidationError("`yj-spacing` must not be None")
    if not isinstance(params["yj-spacing"], (float, int)):
        raise StyxValidationError(f'`yj-spacing` has the wrong type: Received `{type(params.get("yj-spacing", None))}` expected `float`')
    if params.get("yk-spacing", None) is None:
        raise StyxValidationError("`yk-spacing` must not be None")
    if not isinstance(params["yk-spacing"], (float, int)):
        raise StyxValidationError(f'`yk-spacing` has the wrong type: Received `{type(params.get("yk-spacing", None))}` expected `float`')
    if params.get("y-offset", None) is None:
        raise StyxValidationError("`y-offset` must not be None")
    if not isinstance(params["y-offset"], (float, int)):
        raise StyxValidationError(f'`y-offset` has the wrong type: Received `{type(params.get("y-offset", None))}` expected `float`')
    if params.get("zi-spacing", None) is None:
        raise StyxValidationError("`zi-spacing` must not be None")
    if not isinstance(params["zi-spacing"], (float, int)):
        raise StyxValidationError(f'`zi-spacing` has the wrong type: Received `{type(params.get("zi-spacing", None))}` expected `float`')
    if params.get("zj-spacing", None) is None:
        raise StyxValidationError("`zj-spacing` must not be None")
    if not isinstance(params["zj-spacing"], (float, int)):
        raise StyxValidationError(f'`zj-spacing` has the wrong type: Received `{type(params.get("zj-spacing", None))}` expected `float`')
    if params.get("zk-spacing", None) is None:
        raise StyxValidationError("`zk-spacing` must not be None")
    if not isinstance(params["zk-spacing"], (float, int)):
        raise StyxValidationError(f'`zk-spacing` has the wrong type: Received `{type(params.get("zk-spacing", None))}` expected `float`')
    if params.get("z-offset", None) is None:
        raise StyxValidationError("`z-offset` must not be None")
    if not isinstance(params["z-offset"], (float, int)):
        raise StyxValidationError(f'`z-offset` has the wrong type: Received `{type(params.get("z-offset", None))}` expected `float`')


def volume_set_space_sform_cargs(
    params: VolumeSetSpaceSformParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-sform",
        str(params.get("xi-spacing", None)),
        str(params.get("xj-spacing", None)),
        str(params.get("xk-spacing", None)),
        str(params.get("x-offset", None)),
        str(params.get("yi-spacing", None)),
        str(params.get("yj-spacing", None)),
        str(params.get("yk-spacing", None)),
        str(params.get("y-offset", None)),
        str(params.get("zi-spacing", None)),
        str(params.get("zj-spacing", None)),
        str(params.get("zk-spacing", None)),
        str(params.get("z-offset", None))
    ])
    return cargs


def volume_set_space_file(
    volume_ref: str,
    ignore_dims: bool = False,
) -> VolumeSetSpaceFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_ref: volume file to use for reference space.
        ignore_dims: copy the spacing info even if the dimensions don't match.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "file",
        "volume-ref": volume_ref,
        "ignore-dims": ignore_dims,
    }
    return params


def volume_set_space_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeSetSpaceFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-ref", None) is None:
        raise StyxValidationError("`volume-ref` must not be None")
    if not isinstance(params["volume-ref"], str):
        raise StyxValidationError(f'`volume-ref` has the wrong type: Received `{type(params.get("volume-ref", None))}` expected `str`')
    if params.get("ignore-dims", False) is None:
        raise StyxValidationError("`ignore-dims` must not be None")
    if not isinstance(params["ignore-dims"], bool):
        raise StyxValidationError(f'`ignore-dims` has the wrong type: Received `{type(params.get("ignore-dims", False))}` expected `bool`')


def volume_set_space_file_cargs(
    params: VolumeSetSpaceFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-file",
        params.get("volume-ref", None)
    ])
    if params.get("ignore-dims", False):
        cargs.append("-ignore-dims")
    return cargs


class VolumeSetSpaceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeSetSpaceParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_set_space_params(
    volume_in: InputPathType,
    volume_out: str,
    plumb: VolumeSetSpacePlumbParamsDict | None = None,
    sform: VolumeSetSpaceSformParamsDict | None = None,
    file: VolumeSetSpaceFileParamsDict | None = None,
) -> VolumeSetSpaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        volume_out: output - the output volume.
        plumb: set via axis order and spacing/offset.
        sform: set via a nifti sform.
        file: copy spacing info from volume file with matching dimensions.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-set-space",
        "volume-in": volume_in,
        "volume-out": volume_out,
    }
    if plumb is not None:
        params["plumb"] = plumb
    if sform is not None:
        params["sform"] = sform
    if file is not None:
        params["file"] = file
    return params


def volume_set_space_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeSetSpaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("plumb", None) is not None:
        volume_set_space_plumb_validate(params["plumb"])
    if params.get("sform", None) is not None:
        volume_set_space_sform_validate(params["sform"])
    if params.get("file", None) is not None:
        volume_set_space_file_validate(params["file"])


def volume_set_space_cargs(
    params: VolumeSetSpaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-set-space"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("plumb", None) is not None or params.get("sform", None) is not None or params.get("file", None) is not None:
        cargs.extend([
            *(volume_set_space_plumb_cargs(params.get("plumb", None), execution) if (params.get("plumb", None) is not None) else []),
            *(volume_set_space_sform_cargs(params.get("sform", None), execution) if (params.get("sform", None) is not None) else []),
            *(volume_set_space_file_cargs(params.get("file", None), execution) if (params.get("file", None) is not None) else [])
        ])
    return cargs


def volume_set_space_outputs(
    params: VolumeSetSpaceParamsDict,
    execution: Execution,
) -> VolumeSetSpaceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeSetSpaceOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_set_space_execute(
    params: VolumeSetSpaceParamsDict,
    runner: Runner | None = None,
) -> VolumeSetSpaceOutputs:
    """
    CHANGE VOLUME SPACE INFORMATION.
    
    Writes a copy of the volume file, with the spacing information changed as
    specified. No reordering of the voxel data occurs, see -volume-reorient to
    change the volume indexing order and reorder the voxels to match. Exactly
    one of -plumb, -sform, or -file must be specified.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeSetSpaceOutputs`).
    """
    volume_set_space_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_SET_SPACE_METADATA)
    params = execution.params(params)
    cargs = volume_set_space_cargs(params, execution)
    ret = volume_set_space_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_set_space(
    volume_in: InputPathType,
    volume_out: str,
    plumb: VolumeSetSpacePlumbParamsDict | None = None,
    sform: VolumeSetSpaceSformParamsDict | None = None,
    file: VolumeSetSpaceFileParamsDict | None = None,
    runner: Runner | None = None,
) -> VolumeSetSpaceOutputs:
    """
    CHANGE VOLUME SPACE INFORMATION.
    
    Writes a copy of the volume file, with the spacing information changed as
    specified. No reordering of the voxel data occurs, see -volume-reorient to
    change the volume indexing order and reorder the voxels to match. Exactly
    one of -plumb, -sform, or -file must be specified.
    
    Args:
        volume_in: the input volume.
        volume_out: output - the output volume.
        plumb: set via axis order and spacing/offset.
        sform: set via a nifti sform.
        file: copy spacing info from volume file with matching dimensions.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeSetSpaceOutputs`).
    """
    params = volume_set_space_params(
        volume_in=volume_in,
        volume_out=volume_out,
        plumb=plumb,
        sform=sform,
        file=file,
    )
    return volume_set_space_execute(params, runner)


__all__ = [
    "VOLUME_SET_SPACE_METADATA",
    "VolumeSetSpaceFileParamsDict",
    "VolumeSetSpaceFileParamsDictTagged",
    "VolumeSetSpaceOutputs",
    "VolumeSetSpaceParamsDict",
    "VolumeSetSpaceParamsDictTagged",
    "VolumeSetSpacePlumbParamsDict",
    "VolumeSetSpacePlumbParamsDictTagged",
    "VolumeSetSpaceSformParamsDict",
    "VolumeSetSpaceSformParamsDictTagged",
    "volume_set_space",
    "volume_set_space_execute",
    "volume_set_space_file",
    "volume_set_space_params",
    "volume_set_space_plumb",
    "volume_set_space_sform",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_DILATE_METADATA = Metadata(
    id="224f2fa95412d16b7d3853836e3c088fb81a4158.workbench",
    name="cifti-dilate",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiDilateLeftSurfaceParamsDictNoTag = typing.TypedDict('_CiftiDilateLeftSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateLeftSurfaceParamsDictTagged = typing.TypedDict('CiftiDilateLeftSurfaceParamsDictTagged', {
    "@type": typing.Literal["left-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateLeftSurfaceParamsDict = _CiftiDilateLeftSurfaceParamsDictNoTag | CiftiDilateLeftSurfaceParamsDictTagged


_CiftiDilateRightSurfaceParamsDictNoTag = typing.TypedDict('_CiftiDilateRightSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateRightSurfaceParamsDictTagged = typing.TypedDict('CiftiDilateRightSurfaceParamsDictTagged', {
    "@type": typing.Literal["right-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateRightSurfaceParamsDict = _CiftiDilateRightSurfaceParamsDictNoTag | CiftiDilateRightSurfaceParamsDictTagged


_CiftiDilateCerebellumSurfaceParamsDictNoTag = typing.TypedDict('_CiftiDilateCerebellumSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateCerebellumSurfaceParamsDictTagged = typing.TypedDict('CiftiDilateCerebellumSurfaceParamsDictTagged', {
    "@type": typing.Literal["cerebellum-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiDilateCerebellumSurfaceParamsDict = _CiftiDilateCerebellumSurfaceParamsDictNoTag | CiftiDilateCerebellumSurfaceParamsDictTagged


_CiftiDilateParamsDictNoTag = typing.TypedDict('_CiftiDilateParamsDictNoTag', {
    "cifti-in": InputPathType,
    "direction": str,
    "surface-distance": float,
    "volume-distance": float,
    "cifti-out": str,
    "left-surface": typing.NotRequired[CiftiDilateLeftSurfaceParamsDict | None],
    "right-surface": typing.NotRequired[CiftiDilateRightSurfaceParamsDict | None],
    "cerebellum-surface": typing.NotRequired[CiftiDilateCerebellumSurfaceParamsDict | None],
    "roi-cifti": typing.NotRequired[InputPathType | None],
    "legacy-mode": bool,
    "merged-volume": bool,
    "nearest": bool,
})
CiftiDilateParamsDictTagged = typing.TypedDict('CiftiDilateParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-dilate"],
    "cifti-in": InputPathType,
    "direction": str,
    "surface-distance": float,
    "volume-distance": float,
    "cifti-out": str,
    "left-surface": typing.NotRequired[CiftiDilateLeftSurfaceParamsDict | None],
    "right-surface": typing.NotRequired[CiftiDilateRightSurfaceParamsDict | None],
    "cerebellum-surface": typing.NotRequired[CiftiDilateCerebellumSurfaceParamsDict | None],
    "roi-cifti": typing.NotRequired[InputPathType | None],
    "legacy-mode": bool,
    "merged-volume": bool,
    "nearest": bool,
})
CiftiDilateParamsDict = _CiftiDilateParamsDictNoTag | CiftiDilateParamsDictTagged


def cifti_dilate_left_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiDilateLeftSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the left surface file.
        area_metric: vertex areas to use instead of computing them from the\
            left surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_dilate_left_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiDilateLeftSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_dilate_left_surface_cargs(
    params: CiftiDilateLeftSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-left-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def cifti_dilate_right_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiDilateRightSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the right surface file.
        area_metric: vertex areas to use instead of computing them from the\
            right surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_dilate_right_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiDilateRightSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_dilate_right_surface_cargs(
    params: CiftiDilateRightSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-right-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def cifti_dilate_cerebellum_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiDilateCerebellumSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the cerebellum surface file.
        area_metric: vertex areas to use instead of computing them from the\
            cerebellum surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_dilate_cerebellum_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiDilateCerebellumSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_dilate_cerebellum_surface_cargs(
    params: CiftiDilateCerebellumSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-cerebellum-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


class CiftiDilateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiDilateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_dilate_params(
    cifti_in: InputPathType,
    direction: str,
    surface_distance: float,
    volume_distance: float,
    cifti_out: str,
    left_surface: CiftiDilateLeftSurfaceParamsDict | None = None,
    right_surface: CiftiDilateRightSurfaceParamsDict | None = None,
    cerebellum_surface: CiftiDilateCerebellumSurfaceParamsDict | None = None,
    roi_cifti: InputPathType | None = None,
    legacy_mode: bool = False,
    merged_volume: bool = False,
    nearest: bool = False,
) -> CiftiDilateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        direction: which dimension to dilate along, ROW or COLUMN.
        surface_distance: the distance to dilate on surfaces, in mm.
        volume_distance: the distance to dilate in the volume, in mm.
        cifti_out: the output cifti file.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        roi_cifti: specify an roi of brainordinates to overwrite, rather than\
            zeros\
            \
            cifti dscalar or dtseries file, positive values denote\
            brainordinates to have their values replaced.
        legacy_mode: use the math from v1.3.2 and earlier for weighted dilation.
        merged_volume: treat volume components as if they were a single\
            component.
        nearest: use nearest good value instead of a weighted average.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-dilate",
        "cifti-in": cifti_in,
        "direction": direction,
        "surface-distance": surface_distance,
        "volume-distance": volume_distance,
        "cifti-out": cifti_out,
        "legacy-mode": legacy_mode,
        "merged-volume": merged_volume,
        "nearest": nearest,
    }
    if left_surface is not None:
        params["left-surface"] = left_surface
    if right_surface is not None:
        params["right-surface"] = right_surface
    if cerebellum_surface is not None:
        params["cerebellum-surface"] = cerebellum_surface
    if roi_cifti is not None:
        params["roi-cifti"] = roi_cifti
    return params


def cifti_dilate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiDilateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("surface-distance", None) is None:
        raise StyxValidationError("`surface-distance` must not be None")
    if not isinstance(params["surface-distance"], (float, int)):
        raise StyxValidationError(f'`surface-distance` has the wrong type: Received `{type(params.get("surface-distance", None))}` expected `float`')
    if params.get("volume-distance", None) is None:
        raise StyxValidationError("`volume-distance` must not be None")
    if not isinstance(params["volume-distance"], (float, int)):
        raise StyxValidationError(f'`volume-distance` has the wrong type: Received `{type(params.get("volume-distance", None))}` expected `float`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("left-surface", None) is not None:
        cifti_dilate_left_surface_validate(params["left-surface"])
    if params.get("right-surface", None) is not None:
        cifti_dilate_right_surface_validate(params["right-surface"])
    if params.get("cerebellum-surface", None) is not None:
        cifti_dilate_cerebellum_surface_validate(params["cerebellum-surface"])
    if params.get("roi-cifti", None) is not None:
        if not isinstance(params["roi-cifti"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-cifti` has the wrong type: Received `{type(params.get("roi-cifti", None))}` expected `InputPathType | None`')
    if params.get("legacy-mode", False) is None:
        raise StyxValidationError("`legacy-mode` must not be None")
    if not isinstance(params["legacy-mode"], bool):
        raise StyxValidationError(f'`legacy-mode` has the wrong type: Received `{type(params.get("legacy-mode", False))}` expected `bool`')
    if params.get("merged-volume", False) is None:
        raise StyxValidationError("`merged-volume` must not be None")
    if not isinstance(params["merged-volume"], bool):
        raise StyxValidationError(f'`merged-volume` has the wrong type: Received `{type(params.get("merged-volume", False))}` expected `bool`')
    if params.get("nearest", False) is None:
        raise StyxValidationError("`nearest` must not be None")
    if not isinstance(params["nearest"], bool):
        raise StyxValidationError(f'`nearest` has the wrong type: Received `{type(params.get("nearest", False))}` expected `bool`')


def cifti_dilate_cargs(
    params: CiftiDilateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-dilate"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(params.get("direction", None))
    cargs.append(str(params.get("surface-distance", None)))
    cargs.append(str(params.get("volume-distance", None)))
    cargs.append(params.get("cifti-out", None))
    if params.get("left-surface", None) is not None or params.get("right-surface", None) is not None or params.get("cerebellum-surface", None) is not None:
        cargs.extend([
            *(cifti_dilate_left_surface_cargs(params.get("left-surface", None), execution) if (params.get("left-surface", None) is not None) else []),
            *(cifti_dilate_right_surface_cargs(params.get("right-surface", None), execution) if (params.get("right-surface", None) is not None) else []),
            *(cifti_dilate_cerebellum_surface_cargs(params.get("cerebellum-surface", None), execution) if (params.get("cerebellum-surface", None) is not None) else [])
        ])
    if params.get("roi-cifti", None) is not None:
        cargs.extend([
            "-bad-brainordinate-roi",
            execution.input_file(params.get("roi-cifti", None))
        ])
    if params.get("legacy-mode", False):
        cargs.append("-legacy-mode")
    if params.get("merged-volume", False):
        cargs.append("-merged-volume")
    if params.get("nearest", False):
        cargs.append("-nearest")
    return cargs


def cifti_dilate_outputs(
    params: CiftiDilateParamsDict,
    execution: Execution,
) -> CiftiDilateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiDilateOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_dilate_execute(
    params: CiftiDilateParamsDict,
    runner: Runner | None = None,
) -> CiftiDilateOutputs:
    """
    DILATE A CIFTI FILE.
    
    For all data values designated as bad, if they neighbor a good value or are
    within the specified distance of a good value in the same kind of model,
    replace the value with a distance weighted average of nearby good values,
    otherwise set the value to zero. If -nearest is specified, it will use the
    value from the closest good value within range instead of a weighted
    average. When the input file contains label data, nearest dilation is used
    on the surface, and weighted popularity is used in the volume.
    
    The -*-corrected-areas options are intended for dilating on group average
    surfaces, but it is only an approximate correction for the reduction of
    structure in a group average surface.
    
    If -bad-brainordinate-roi is specified, all values, including those with
    value zero, are good, except for locations with a positive value in the ROI.
    If it is not specified, only values equal to zero are bad.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiDilateOutputs`).
    """
    cifti_dilate_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_DILATE_METADATA)
    params = execution.params(params)
    cargs = cifti_dilate_cargs(params, execution)
    ret = cifti_dilate_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_dilate(
    cifti_in: InputPathType,
    direction: str,
    surface_distance: float,
    volume_distance: float,
    cifti_out: str,
    left_surface: CiftiDilateLeftSurfaceParamsDict | None = None,
    right_surface: CiftiDilateRightSurfaceParamsDict | None = None,
    cerebellum_surface: CiftiDilateCerebellumSurfaceParamsDict | None = None,
    roi_cifti: InputPathType | None = None,
    legacy_mode: bool = False,
    merged_volume: bool = False,
    nearest: bool = False,
    runner: Runner | None = None,
) -> CiftiDilateOutputs:
    """
    DILATE A CIFTI FILE.
    
    For all data values designated as bad, if they neighbor a good value or are
    within the specified distance of a good value in the same kind of model,
    replace the value with a distance weighted average of nearby good values,
    otherwise set the value to zero. If -nearest is specified, it will use the
    value from the closest good value within range instead of a weighted
    average. When the input file contains label data, nearest dilation is used
    on the surface, and weighted popularity is used in the volume.
    
    The -*-corrected-areas options are intended for dilating on group average
    surfaces, but it is only an approximate correction for the reduction of
    structure in a group average surface.
    
    If -bad-brainordinate-roi is specified, all values, including those with
    value zero, are good, except for locations with a positive value in the ROI.
    If it is not specified, only values equal to zero are bad.
    
    Args:
        cifti_in: the input cifti file.
        direction: which dimension to dilate along, ROW or COLUMN.
        surface_distance: the distance to dilate on surfaces, in mm.
        volume_distance: the distance to dilate in the volume, in mm.
        cifti_out: the output cifti file.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        roi_cifti: specify an roi of brainordinates to overwrite, rather than\
            zeros\
            \
            cifti dscalar or dtseries file, positive values denote\
            brainordinates to have their values replaced.
        legacy_mode: use the math from v1.3.2 and earlier for weighted dilation.
        merged_volume: treat volume components as if they were a single\
            component.
        nearest: use nearest good value instead of a weighted average.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiDilateOutputs`).
    """
    params = cifti_dilate_params(
        cifti_in=cifti_in,
        direction=direction,
        surface_distance=surface_distance,
        volume_distance=volume_distance,
        cifti_out=cifti_out,
        left_surface=left_surface,
        right_surface=right_surface,
        cerebellum_surface=cerebellum_surface,
        roi_cifti=roi_cifti,
        legacy_mode=legacy_mode,
        merged_volume=merged_volume,
        nearest=nearest,
    )
    return cifti_dilate_execute(params, runner)


__all__ = [
    "CIFTI_DILATE_METADATA",
    "CiftiDilateCerebellumSurfaceParamsDict",
    "CiftiDilateCerebellumSurfaceParamsDictTagged",
    "CiftiDilateLeftSurfaceParamsDict",
    "CiftiDilateLeftSurfaceParamsDictTagged",
    "CiftiDilateOutputs",
    "CiftiDilateParamsDict",
    "CiftiDilateParamsDictTagged",
    "CiftiDilateRightSurfaceParamsDict",
    "CiftiDilateRightSurfaceParamsDictTagged",
    "cifti_dilate",
    "cifti_dilate_cerebellum_surface",
    "cifti_dilate_execute",
    "cifti_dilate_left_surface",
    "cifti_dilate_params",
    "cifti_dilate_right_surface",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_INFLATION_METADATA = Metadata(
    id="9461f44d7d7d0b826502df0acb5b89588faca185.workbench",
    name="surface-inflation",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceInflationParamsDictNoTag = typing.TypedDict('_SurfaceInflationParamsDictNoTag', {
    "anatomical-surface-in": InputPathType,
    "surface-in": InputPathType,
    "number-of-smoothing-cycles": int,
    "smoothing-strength": float,
    "smoothing-iterations": int,
    "inflation-factor": float,
    "surface-out": str,
})
SurfaceInflationParamsDictTagged = typing.TypedDict('SurfaceInflationParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-inflation"],
    "anatomical-surface-in": InputPathType,
    "surface-in": InputPathType,
    "number-of-smoothing-cycles": int,
    "smoothing-strength": float,
    "smoothing-iterations": int,
    "inflation-factor": float,
    "surface-out": str,
})
SurfaceInflationParamsDict = _SurfaceInflationParamsDictNoTag | SurfaceInflationParamsDictTagged


class SurfaceInflationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceInflationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """output surface file"""


def surface_inflation_params(
    anatomical_surface_in: InputPathType,
    surface_in: InputPathType,
    number_of_smoothing_cycles: int,
    smoothing_strength: float,
    smoothing_iterations: int,
    inflation_factor: float,
    surface_out: str,
) -> SurfaceInflationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        anatomical_surface_in: the anatomical surface.
        surface_in: the surface file to inflate.
        number_of_smoothing_cycles: number of smoothing cycles.
        smoothing_strength: smoothing strength (ranges [0.0 - 1.0]).
        smoothing_iterations: smoothing iterations.
        inflation_factor: inflation factor.
        surface_out: output surface file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-inflation",
        "anatomical-surface-in": anatomical_surface_in,
        "surface-in": surface_in,
        "number-of-smoothing-cycles": number_of_smoothing_cycles,
        "smoothing-strength": smoothing_strength,
        "smoothing-iterations": smoothing_iterations,
        "inflation-factor": inflation_factor,
        "surface-out": surface_out,
    }
    return params


def surface_inflation_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceInflationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("anatomical-surface-in", None) is None:
        raise StyxValidationError("`anatomical-surface-in` must not be None")
    if not isinstance(params["anatomical-surface-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`anatomical-surface-in` has the wrong type: Received `{type(params.get("anatomical-surface-in", None))}` expected `InputPathType`')
    if params.get("surface-in", None) is None:
        raise StyxValidationError("`surface-in` must not be None")
    if not isinstance(params["surface-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-in` has the wrong type: Received `{type(params.get("surface-in", None))}` expected `InputPathType`')
    if params.get("number-of-smoothing-cycles", None) is None:
        raise StyxValidationError("`number-of-smoothing-cycles` must not be None")
    if not isinstance(params["number-of-smoothing-cycles"], int):
        raise StyxValidationError(f'`number-of-smoothing-cycles` has the wrong type: Received `{type(params.get("number-of-smoothing-cycles", None))}` expected `int`')
    if params.get("smoothing-strength", None) is None:
        raise StyxValidationError("`smoothing-strength` must not be None")
    if not isinstance(params["smoothing-strength"], (float, int)):
        raise StyxValidationError(f'`smoothing-strength` has the wrong type: Received `{type(params.get("smoothing-strength", None))}` expected `float`')
    if params.get("smoothing-iterations", None) is None:
        raise StyxValidationError("`smoothing-iterations` must not be None")
    if not isinstance(params["smoothing-iterations"], int):
        raise StyxValidationError(f'`smoothing-iterations` has the wrong type: Received `{type(params.get("smoothing-iterations", None))}` expected `int`')
    if params.get("inflation-factor", None) is None:
        raise StyxValidationError("`inflation-factor` must not be None")
    if not isinstance(params["inflation-factor"], (float, int)):
        raise StyxValidationError(f'`inflation-factor` has the wrong type: Received `{type(params.get("inflation-factor", None))}` expected `float`')
    if params.get("surface-out", None) is None:
        raise StyxValidationError("`surface-out` must not be None")
    if not isinstance(params["surface-out"], str):
        raise StyxValidationError(f'`surface-out` has the wrong type: Received `{type(params.get("surface-out", None))}` expected `str`')


def surface_inflation_cargs(
    params: SurfaceInflationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-inflation"
    ])
    cargs.append(execution.input_file(params.get("anatomical-surface-in", None)))
    cargs.append(execution.input_file(params.get("surface-in", None)))
    cargs.append(str(params.get("number-of-smoothing-cycles", None)))
    cargs.append(str(params.get("smoothing-strength", None)))
    cargs.append(str(params.get("smoothing-iterations", None)))
    cargs.append(str(params.get("inflation-factor", None)))
    cargs.append(params.get("surface-out", None))
    return cargs


def surface_inflation_outputs(
    params: SurfaceInflationParamsDict,
    execution: Execution,
) -> SurfaceInflationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceInflationOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(params.get("surface-out", None)),
    )
    return ret


def surface_inflation_execute(
    params: SurfaceInflationParamsDict,
    runner: Runner | None = None,
) -> SurfaceInflationOutputs:
    """
    SURFACE INFLATION.
    
    Inflate a surface by performing cycles that consist of smoothing followed by
    inflation (to correct shrinkage caused by smoothing).
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceInflationOutputs`).
    """
    surface_inflation_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_INFLATION_METADATA)
    params = execution.params(params)
    cargs = surface_inflation_cargs(params, execution)
    ret = surface_inflation_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_inflation(
    anatomical_surface_in: InputPathType,
    surface_in: InputPathType,
    number_of_smoothing_cycles: int,
    smoothing_strength: float,
    smoothing_iterations: int,
    inflation_factor: float,
    surface_out: str,
    runner: Runner | None = None,
) -> SurfaceInflationOutputs:
    """
    SURFACE INFLATION.
    
    Inflate a surface by performing cycles that consist of smoothing followed by
    inflation (to correct shrinkage caused by smoothing).
    
    Args:
        anatomical_surface_in: the anatomical surface.
        surface_in: the surface file to inflate.
        number_of_smoothing_cycles: number of smoothing cycles.
        smoothing_strength: smoothing strength (ranges [0.0 - 1.0]).
        smoothing_iterations: smoothing iterations.
        inflation_factor: inflation factor.
        surface_out: output surface file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceInflationOutputs`).
    """
    params = surface_inflation_params(
        anatomical_surface_in=anatomical_surface_in,
        surface_in=surface_in,
        number_of_smoothing_cycles=number_of_smoothing_cycles,
        smoothing_strength=smoothing_strength,
        smoothing_iterations=smoothing_iterations,
        inflation_factor=inflation_factor,
        surface_out=surface_out,
    )
    return surface_inflation_execute(params, runner)


__all__ = [
    "SURFACE_INFLATION_METADATA",
    "SurfaceInflationOutputs",
    "SurfaceInflationParamsDict",
    "SurfaceInflationParamsDictTagged",
    "surface_inflation",
    "surface_inflation_execute",
    "surface_inflation_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_REGRESSION_METADATA = Metadata(
    id="112cf7b9fc8784489485c6275842f1d2dadef8d8.workbench",
    name="metric-regression",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricRegressionRemoveParamsDictNoTag = typing.TypedDict('_MetricRegressionRemoveParamsDictNoTag', {
    "metric": InputPathType,
    "column": typing.NotRequired[str | None],
})
MetricRegressionRemoveParamsDictTagged = typing.TypedDict('MetricRegressionRemoveParamsDictTagged', {
    "@type": typing.Literal["remove"],
    "metric": InputPathType,
    "column": typing.NotRequired[str | None],
})
MetricRegressionRemoveParamsDict = _MetricRegressionRemoveParamsDictNoTag | MetricRegressionRemoveParamsDictTagged


_MetricRegressionKeepParamsDictNoTag = typing.TypedDict('_MetricRegressionKeepParamsDictNoTag', {
    "metric": InputPathType,
    "column": typing.NotRequired[str | None],
})
MetricRegressionKeepParamsDictTagged = typing.TypedDict('MetricRegressionKeepParamsDictTagged', {
    "@type": typing.Literal["keep"],
    "metric": InputPathType,
    "column": typing.NotRequired[str | None],
})
MetricRegressionKeepParamsDict = _MetricRegressionKeepParamsDictNoTag | MetricRegressionKeepParamsDictTagged


_MetricRegressionParamsDictNoTag = typing.TypedDict('_MetricRegressionParamsDictNoTag', {
    "metric-in": InputPathType,
    "metric-out": str,
    "remove": typing.NotRequired[list[MetricRegressionRemoveParamsDict] | None],
    "keep": typing.NotRequired[list[MetricRegressionKeepParamsDict] | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricRegressionParamsDictTagged = typing.TypedDict('MetricRegressionParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-regression"],
    "metric-in": InputPathType,
    "metric-out": str,
    "remove": typing.NotRequired[list[MetricRegressionRemoveParamsDict] | None],
    "keep": typing.NotRequired[list[MetricRegressionKeepParamsDict] | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricRegressionParamsDict = _MetricRegressionParamsDictNoTag | MetricRegressionParamsDictTagged


def metric_regression_remove(
    metric: InputPathType,
    column: str | None = None,
) -> MetricRegressionRemoveParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file to use.
        column: select a column to use, rather than all\
            \
            the column number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "remove",
        "metric": metric,
    }
    if column is not None:
        params["column"] = column
    return params


def metric_regression_remove_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricRegressionRemoveParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')


def metric_regression_remove_cargs(
    params: MetricRegressionRemoveParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-remove",
        execution.input_file(params.get("metric", None))
    ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-remove-column",
            params.get("column", None)
        ])
    return cargs


def metric_regression_keep(
    metric: InputPathType,
    column: str | None = None,
) -> MetricRegressionKeepParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file to use.
        column: select a column to use, rather than all\
            \
            the column number or name.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "keep",
        "metric": metric,
    }
    if column is not None:
        params["column"] = column
    return params


def metric_regression_keep_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricRegressionKeepParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')


def metric_regression_keep_cargs(
    params: MetricRegressionKeepParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-keep",
        execution.input_file(params.get("metric", None))
    ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-keep-column",
            params.get("column", None)
        ])
    return cargs


class MetricRegressionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricRegressionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_regression_params(
    metric_in: InputPathType,
    metric_out: str,
    remove: list[MetricRegressionRemoveParamsDict] | None = None,
    keep: list[MetricRegressionKeepParamsDict] | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
) -> MetricRegressionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric_in: the metric to regress from.
        metric_out: the output metric.
        remove: specify a metric to regress out.
        keep: specify a metric to include in regression, but not remove.
        column: select a single column to regress from\
            \
            the column number or name.
        roi_metric: only regress inside an roi\
            \
            the area to use for regression, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-regression",
        "metric-in": metric_in,
        "metric-out": metric_out,
    }
    if remove is not None:
        params["remove"] = remove
    if keep is not None:
        params["keep"] = keep
    if column is not None:
        params["column"] = column
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def metric_regression_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricRegressionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric-in", None) is None:
        raise StyxValidationError("`metric-in` must not be None")
    if not isinstance(params["metric-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric-in` has the wrong type: Received `{type(params.get("metric-in", None))}` expected `InputPathType`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("remove", None) is not None:
        if not isinstance(params["remove"], list):
            raise StyxValidationError(f'`remove` has the wrong type: Received `{type(params.get("remove", None))}` expected `list[MetricRegressionRemoveParamsDict] | None`')
        for e in params["remove"]:
            metric_regression_remove_validate(e)
    if params.get("keep", None) is not None:
        if not isinstance(params["keep"], list):
            raise StyxValidationError(f'`keep` has the wrong type: Received `{type(params.get("keep", None))}` expected `list[MetricRegressionKeepParamsDict] | None`')
        for e in params["keep"]:
            metric_regression_keep_validate(e)
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def metric_regression_cargs(
    params: MetricRegressionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-regression"
    ])
    cargs.append(execution.input_file(params.get("metric-in", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("remove", None) is not None or params.get("keep", None) is not None:
        cargs.extend([
            *([a for c in [metric_regression_remove_cargs(s, execution) for s in params.get("remove", None)] for a in c] if (params.get("remove", None) is not None) else []),
            *([a for c in [metric_regression_keep_cargs(s, execution) for s in params.get("keep", None)] for a in c] if (params.get("keep", None) is not None) else [])
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def metric_regression_outputs(
    params: MetricRegressionParamsDict,
    execution: Execution,
) -> MetricRegressionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricRegressionOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_regression_execute(
    params: MetricRegressionParamsDict,
    runner: Runner | None = None,
) -> MetricRegressionOutputs:
    """
    REGRESS SPATIAL MAP OUT OF A METRIC FILE.
    
    For each regressor, its mean across the surface is subtracted from its data.
    Each input map is then regressed against these, and a constant term. The
    resulting regressed slopes of all regressors specified with -remove are
    multiplied with their respective regressor maps, and these are subtracted
    from the input map.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricRegressionOutputs`).
    """
    metric_regression_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_REGRESSION_METADATA)
    params = execution.params(params)
    cargs = metric_regression_cargs(params, execution)
    ret = metric_regression_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_regression(
    metric_in: InputPathType,
    metric_out: str,
    remove: list[MetricRegressionRemoveParamsDict] | None = None,
    keep: list[MetricRegressionKeepParamsDict] | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> MetricRegressionOutputs:
    """
    REGRESS SPATIAL MAP OUT OF A METRIC FILE.
    
    For each regressor, its mean across the surface is subtracted from its data.
    Each input map is then regressed against these, and a constant term. The
    resulting regressed slopes of all regressors specified with -remove are
    multiplied with their respective regressor maps, and these are subtracted
    from the input map.
    
    Args:
        metric_in: the metric to regress from.
        metric_out: the output metric.
        remove: specify a metric to regress out.
        keep: specify a metric to include in regression, but not remove.
        column: select a single column to regress from\
            \
            the column number or name.
        roi_metric: only regress inside an roi\
            \
            the area to use for regression, as a metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricRegressionOutputs`).
    """
    params = metric_regression_params(
        metric_in=metric_in,
        metric_out=metric_out,
        remove=remove,
        keep=keep,
        column=column,
        roi_metric=roi_metric,
    )
    return metric_regression_execute(params, runner)


__all__ = [
    "METRIC_REGRESSION_METADATA",
    "MetricRegressionKeepParamsDict",
    "MetricRegressionKeepParamsDictTagged",
    "MetricRegressionOutputs",
    "MetricRegressionParamsDict",
    "MetricRegressionParamsDictTagged",
    "MetricRegressionRemoveParamsDict",
    "MetricRegressionRemoveParamsDictTagged",
    "metric_regression",
    "metric_regression_execute",
    "metric_regression_keep",
    "metric_regression_params",
    "metric_regression_remove",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CORRELATION_GRADIENT_METADATA = Metadata(
    id="ee61d770d1b0dbf0c964ee821bf79b84aaac30f5.workbench",
    name="cifti-correlation-gradient",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiCorrelationGradientLeftSurfaceParamsDictNoTag = typing.TypedDict('_CiftiCorrelationGradientLeftSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientLeftSurfaceParamsDictTagged = typing.TypedDict('CiftiCorrelationGradientLeftSurfaceParamsDictTagged', {
    "@type": typing.Literal["left-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientLeftSurfaceParamsDict = _CiftiCorrelationGradientLeftSurfaceParamsDictNoTag | CiftiCorrelationGradientLeftSurfaceParamsDictTagged


_CiftiCorrelationGradientRightSurfaceParamsDictNoTag = typing.TypedDict('_CiftiCorrelationGradientRightSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientRightSurfaceParamsDictTagged = typing.TypedDict('CiftiCorrelationGradientRightSurfaceParamsDictTagged', {
    "@type": typing.Literal["right-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientRightSurfaceParamsDict = _CiftiCorrelationGradientRightSurfaceParamsDictNoTag | CiftiCorrelationGradientRightSurfaceParamsDictTagged


_CiftiCorrelationGradientCerebellumSurfaceParamsDictNoTag = typing.TypedDict('_CiftiCorrelationGradientCerebellumSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientCerebellumSurfaceParamsDictTagged = typing.TypedDict('CiftiCorrelationGradientCerebellumSurfaceParamsDictTagged', {
    "@type": typing.Literal["cerebellum-surface"],
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
})
CiftiCorrelationGradientCerebellumSurfaceParamsDict = _CiftiCorrelationGradientCerebellumSurfaceParamsDictNoTag | CiftiCorrelationGradientCerebellumSurfaceParamsDictTagged


_CiftiCorrelationGradientDoubleCorrelationParamsDictNoTag = typing.TypedDict('_CiftiCorrelationGradientDoubleCorrelationParamsDictNoTag', {
    "covariance-first": bool,
    "no-demean-first": bool,
    "fisher-z-first": bool,
})
CiftiCorrelationGradientDoubleCorrelationParamsDictTagged = typing.TypedDict('CiftiCorrelationGradientDoubleCorrelationParamsDictTagged', {
    "@type": typing.Literal["double-correlation"],
    "covariance-first": bool,
    "no-demean-first": bool,
    "fisher-z-first": bool,
})
CiftiCorrelationGradientDoubleCorrelationParamsDict = _CiftiCorrelationGradientDoubleCorrelationParamsDictNoTag | CiftiCorrelationGradientDoubleCorrelationParamsDictTagged


_CiftiCorrelationGradientParamsDictNoTag = typing.TypedDict('_CiftiCorrelationGradientParamsDictNoTag', {
    "cifti": InputPathType,
    "cifti-out": str,
    "left-surface": typing.NotRequired[CiftiCorrelationGradientLeftSurfaceParamsDict | None],
    "right-surface": typing.NotRequired[CiftiCorrelationGradientRightSurfaceParamsDict | None],
    "cerebellum-surface": typing.NotRequired[CiftiCorrelationGradientCerebellumSurfaceParamsDict | None],
    "double-correlation": typing.NotRequired[CiftiCorrelationGradientDoubleCorrelationParamsDict | None],
    "limit-GB": typing.NotRequired[float | None],
    "distance": typing.NotRequired[float | None],
    "distance": typing.NotRequired[float | None],
    "volume-kernel": typing.NotRequired[float | None],
    "surface-kernel": typing.NotRequired[float | None],
    "covariance": bool,
    "fisher-z": bool,
    "undo-fisher-z": bool,
    "presmooth-fwhm": bool,
})
CiftiCorrelationGradientParamsDictTagged = typing.TypedDict('CiftiCorrelationGradientParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-correlation-gradient"],
    "cifti": InputPathType,
    "cifti-out": str,
    "left-surface": typing.NotRequired[CiftiCorrelationGradientLeftSurfaceParamsDict | None],
    "right-surface": typing.NotRequired[CiftiCorrelationGradientRightSurfaceParamsDict | None],
    "cerebellum-surface": typing.NotRequired[CiftiCorrelationGradientCerebellumSurfaceParamsDict | None],
    "double-correlation": typing.NotRequired[CiftiCorrelationGradientDoubleCorrelationParamsDict | None],
    "limit-GB": typing.NotRequired[float | None],
    "distance": typing.NotRequired[float | None],
    "distance": typing.NotRequired[float | None],
    "volume-kernel": typing.NotRequired[float | None],
    "surface-kernel": typing.NotRequired[float | None],
    "covariance": bool,
    "fisher-z": bool,
    "undo-fisher-z": bool,
    "presmooth-fwhm": bool,
})
CiftiCorrelationGradientParamsDict = _CiftiCorrelationGradientParamsDictNoTag | CiftiCorrelationGradientParamsDictTagged


def cifti_correlation_gradient_left_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiCorrelationGradientLeftSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the left surface file.
        area_metric: vertex areas to use instead of computing them from the\
            left surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_correlation_gradient_left_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCorrelationGradientLeftSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_correlation_gradient_left_surface_cargs(
    params: CiftiCorrelationGradientLeftSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-left-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def cifti_correlation_gradient_right_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiCorrelationGradientRightSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the right surface file.
        area_metric: vertex areas to use instead of computing them from the\
            right surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_correlation_gradient_right_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCorrelationGradientRightSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_correlation_gradient_right_surface_cargs(
    params: CiftiCorrelationGradientRightSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-right-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def cifti_correlation_gradient_cerebellum_surface(
    surface: InputPathType,
    area_metric: InputPathType | None = None,
) -> CiftiCorrelationGradientCerebellumSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the cerebellum surface file.
        area_metric: vertex areas to use instead of computing them from the\
            cerebellum surface\
            \
            the corrected vertex areas, as a metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-surface",
        "surface": surface,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def cifti_correlation_gradient_cerebellum_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCorrelationGradientCerebellumSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')


def cifti_correlation_gradient_cerebellum_surface_cargs(
    params: CiftiCorrelationGradientCerebellumSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-cerebellum-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    return cargs


def cifti_correlation_gradient_double_correlation(
    covariance_first: bool = False,
    no_demean_first: bool = False,
    fisher_z_first: bool = False,
) -> CiftiCorrelationGradientDoubleCorrelationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        covariance_first: instead of correlation for the FIRST operation,\
            compute covariance.
        no_demean_first: instead of correlation for the FIRST operation, do dot\
            product of rows, then normalize by diagonal.
        fisher_z_first: after the FIRST correlation, apply fisher small z\
            transform (ie, artanh).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "double-correlation",
        "covariance-first": covariance_first,
        "no-demean-first": no_demean_first,
        "fisher-z-first": fisher_z_first,
    }
    return params


def cifti_correlation_gradient_double_correlation_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCorrelationGradientDoubleCorrelationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("covariance-first", False) is None:
        raise StyxValidationError("`covariance-first` must not be None")
    if not isinstance(params["covariance-first"], bool):
        raise StyxValidationError(f'`covariance-first` has the wrong type: Received `{type(params.get("covariance-first", False))}` expected `bool`')
    if params.get("no-demean-first", False) is None:
        raise StyxValidationError("`no-demean-first` must not be None")
    if not isinstance(params["no-demean-first"], bool):
        raise StyxValidationError(f'`no-demean-first` has the wrong type: Received `{type(params.get("no-demean-first", False))}` expected `bool`')
    if params.get("fisher-z-first", False) is None:
        raise StyxValidationError("`fisher-z-first` must not be None")
    if not isinstance(params["fisher-z-first"], bool):
        raise StyxValidationError(f'`fisher-z-first` has the wrong type: Received `{type(params.get("fisher-z-first", False))}` expected `bool`')


def cifti_correlation_gradient_double_correlation_cargs(
    params: CiftiCorrelationGradientDoubleCorrelationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-double-correlation")
    if params.get("covariance-first", False):
        cargs.append("-covariance-first")
    if params.get("no-demean-first", False):
        cargs.append("-no-demean-first")
    if params.get("fisher-z-first", False):
        cargs.append("-fisher-z-first")
    return cargs


class CiftiCorrelationGradientOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiCorrelationGradientParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti"""


def cifti_correlation_gradient_params(
    cifti: InputPathType,
    cifti_out: str,
    left_surface: CiftiCorrelationGradientLeftSurfaceParamsDict | None = None,
    right_surface: CiftiCorrelationGradientRightSurfaceParamsDict | None = None,
    cerebellum_surface: CiftiCorrelationGradientCerebellumSurfaceParamsDict | None = None,
    double_correlation: CiftiCorrelationGradientDoubleCorrelationParamsDict | None = None,
    limit_gb: float | None = None,
    distance: float | None = None,
    distance_: float | None = None,
    volume_kernel: float | None = None,
    surface_kernel: float | None = None,
    covariance: bool = False,
    fisher_z: bool = False,
    undo_fisher_z: bool = False,
    presmooth_fwhm: bool = False,
) -> CiftiCorrelationGradientParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti: the input cifti.
        cifti_out: the output cifti.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        double_correlation: do two correlations before taking the gradient.
        limit_gb: restrict memory usage\
            \
            memory limit in gigabytes.
        distance: exclude voxels near each seed voxel from computation\
            \
            distance from seed voxel for the exclusion zone, in mm.
        distance_: exclude vertices near each seed vertex from computation\
            \
            geodesic distance from seed vertex for the exclusion zone, in mm.
        volume_kernel: smooth the volume before computing the gradient\
            \
            the size of the gaussian volume smoothing kernel in mm, as sigma by\
            default.
        surface_kernel: smooth on the surface before computing the gradient\
            \
            the size of the gaussian surface smoothing kernel in mm, as sigma\
            by default.
        covariance: compute covariance instead of correlation.
        fisher_z: apply the fisher small z transform to the correlations before\
            taking the gradient.
        undo_fisher_z: apply the inverse fisher small z transform to the input.
        presmooth_fwhm: smoothing kernel sizes are FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-correlation-gradient",
        "cifti": cifti,
        "cifti-out": cifti_out,
        "covariance": covariance,
        "fisher-z": fisher_z,
        "undo-fisher-z": undo_fisher_z,
        "presmooth-fwhm": presmooth_fwhm,
    }
    if left_surface is not None:
        params["left-surface"] = left_surface
    if right_surface is not None:
        params["right-surface"] = right_surface
    if cerebellum_surface is not None:
        params["cerebellum-surface"] = cerebellum_surface
    if double_correlation is not None:
        params["double-correlation"] = double_correlation
    if limit_gb is not None:
        params["limit-GB"] = limit_gb
    if distance is not None:
        params["distance"] = distance
    if distance_ is not None:
        params["distance"] = distance_
    if volume_kernel is not None:
        params["volume-kernel"] = volume_kernel
    if surface_kernel is not None:
        params["surface-kernel"] = surface_kernel
    return params


def cifti_correlation_gradient_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCorrelationGradientParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti", None) is None:
        raise StyxValidationError("`cifti` must not be None")
    if not isinstance(params["cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `InputPathType`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("left-surface", None) is not None:
        cifti_correlation_gradient_left_surface_validate(params["left-surface"])
    if params.get("right-surface", None) is not None:
        cifti_correlation_gradient_right_surface_validate(params["right-surface"])
    if params.get("cerebellum-surface", None) is not None:
        cifti_correlation_gradient_cerebellum_surface_validate(params["cerebellum-surface"])
    if params.get("double-correlation", None) is not None:
        cifti_correlation_gradient_double_correlation_validate(params["double-correlation"])
    if params.get("limit-GB", None) is not None:
        if not isinstance(params["limit-GB"], (float, int)):
            raise StyxValidationError(f'`limit-GB` has the wrong type: Received `{type(params.get("limit-GB", None))}` expected `float | None`')
    if params.get("distance", None) is not None:
        if not isinstance(params["distance"], (float, int)):
            raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float | None`')
    if params.get("distance", None) is not None:
        if not isinstance(params["distance"], (float, int)):
            raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float | None`')
    if params.get("volume-kernel", None) is not None:
        if not isinstance(params["volume-kernel"], (float, int)):
            raise StyxValidationError(f'`volume-kernel` has the wrong type: Received `{type(params.get("volume-kernel", None))}` expected `float | None`')
    if params.get("surface-kernel", None) is not None:
        if not isinstance(params["surface-kernel"], (float, int)):
            raise StyxValidationError(f'`surface-kernel` has the wrong type: Received `{type(params.get("surface-kernel", None))}` expected `float | None`')
    if params.get("covariance", False) is None:
        raise StyxValidationError("`covariance` must not be None")
    if not isinstance(params["covariance"], bool):
        raise StyxValidationError(f'`covariance` has the wrong type: Received `{type(params.get("covariance", False))}` expected `bool`')
    if params.get("fisher-z", False) is None:
        raise StyxValidationError("`fisher-z` must not be None")
    if not isinstance(params["fisher-z"], bool):
        raise StyxValidationError(f'`fisher-z` has the wrong type: Received `{type(params.get("fisher-z", False))}` expected `bool`')
    if params.get("undo-fisher-z", False) is None:
        raise StyxValidationError("`undo-fisher-z` must not be None")
    if not isinstance(params["undo-fisher-z"], bool):
        raise StyxValidationError(f'`undo-fisher-z` has the wrong type: Received `{type(params.get("undo-fisher-z", False))}` expected `bool`')
    if params.get("presmooth-fwhm", False) is None:
        raise StyxValidationError("`presmooth-fwhm` must not be None")
    if not isinstance(params["presmooth-fwhm"], bool):
        raise StyxValidationError(f'`presmooth-fwhm` has the wrong type: Received `{type(params.get("presmooth-fwhm", False))}` expected `bool`')


def cifti_correlation_gradient_cargs(
    params: CiftiCorrelationGradientParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-correlation-gradient"
    ])
    cargs.append(execution.input_file(params.get("cifti", None)))
    cargs.append(params.get("cifti-out", None))
    if params.get("left-surface", None) is not None or params.get("right-surface", None) is not None or params.get("cerebellum-surface", None) is not None or params.get("double-correlation", None) is not None:
        cargs.extend([
            *(cifti_correlation_gradient_left_surface_cargs(params.get("left-surface", None), execution) if (params.get("left-surface", None) is not None) else []),
            *(cifti_correlation_gradient_right_surface_cargs(params.get("right-surface", None), execution) if (params.get("right-surface", None) is not None) else []),
            *(cifti_correlation_gradient_cerebellum_surface_cargs(params.get("cerebellum-surface", None), execution) if (params.get("cerebellum-surface", None) is not None) else []),
            *(cifti_correlation_gradient_double_correlation_cargs(params.get("double-correlation", None), execution) if (params.get("double-correlation", None) is not None) else [])
        ])
    if params.get("limit-GB", None) is not None:
        cargs.extend([
            "-mem-limit",
            str(params.get("limit-GB", None))
        ])
    if params.get("distance", None) is not None:
        cargs.extend([
            "-volume-exclude",
            str(params.get("distance", None))
        ])
    if params.get("distance", None) is not None:
        cargs.extend([
            "-surface-exclude",
            str(params.get("distance", None))
        ])
    if params.get("volume-kernel", None) is not None:
        cargs.extend([
            "-volume-presmooth",
            str(params.get("volume-kernel", None))
        ])
    if params.get("surface-kernel", None) is not None:
        cargs.extend([
            "-surface-presmooth",
            str(params.get("surface-kernel", None))
        ])
    if params.get("covariance", False):
        cargs.append("-covariance")
    if params.get("fisher-z", False):
        cargs.append("-fisher-z")
    if params.get("undo-fisher-z", False):
        cargs.append("-undo-fisher-z")
    if params.get("presmooth-fwhm", False):
        cargs.append("-presmooth-fwhm")
    return cargs


def cifti_correlation_gradient_outputs(
    params: CiftiCorrelationGradientParamsDict,
    execution: Execution,
) -> CiftiCorrelationGradientOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiCorrelationGradientOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_correlation_gradient_execute(
    params: CiftiCorrelationGradientParamsDict,
    runner: Runner | None = None,
) -> CiftiCorrelationGradientOutputs:
    """
    CORRELATE CIFTI ROWS AND TAKE GRADIENT.
    
    For each structure, compute the correlation of the rows in the structure,
    and take the gradients of the resulting rows, then average them. Memory
    limit does not need to be an integer, you may also specify 0 to use as
    little memory as possible (this may be very slow).
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCorrelationGradientOutputs`).
    """
    cifti_correlation_gradient_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CORRELATION_GRADIENT_METADATA)
    params = execution.params(params)
    cargs = cifti_correlation_gradient_cargs(params, execution)
    ret = cifti_correlation_gradient_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_correlation_gradient(
    cifti: InputPathType,
    cifti_out: str,
    left_surface: CiftiCorrelationGradientLeftSurfaceParamsDict | None = None,
    right_surface: CiftiCorrelationGradientRightSurfaceParamsDict | None = None,
    cerebellum_surface: CiftiCorrelationGradientCerebellumSurfaceParamsDict | None = None,
    double_correlation: CiftiCorrelationGradientDoubleCorrelationParamsDict | None = None,
    limit_gb: float | None = None,
    distance: float | None = None,
    distance_: float | None = None,
    volume_kernel: float | None = None,
    surface_kernel: float | None = None,
    covariance: bool = False,
    fisher_z: bool = False,
    undo_fisher_z: bool = False,
    presmooth_fwhm: bool = False,
    runner: Runner | None = None,
) -> CiftiCorrelationGradientOutputs:
    """
    CORRELATE CIFTI ROWS AND TAKE GRADIENT.
    
    For each structure, compute the correlation of the rows in the structure,
    and take the gradients of the resulting rows, then average them. Memory
    limit does not need to be an integer, you may also specify 0 to use as
    little memory as possible (this may be very slow).
    
    Args:
        cifti: the input cifti.
        cifti_out: the output cifti.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        double_correlation: do two correlations before taking the gradient.
        limit_gb: restrict memory usage\
            \
            memory limit in gigabytes.
        distance: exclude voxels near each seed voxel from computation\
            \
            distance from seed voxel for the exclusion zone, in mm.
        distance_: exclude vertices near each seed vertex from computation\
            \
            geodesic distance from seed vertex for the exclusion zone, in mm.
        volume_kernel: smooth the volume before computing the gradient\
            \
            the size of the gaussian volume smoothing kernel in mm, as sigma by\
            default.
        surface_kernel: smooth on the surface before computing the gradient\
            \
            the size of the gaussian surface smoothing kernel in mm, as sigma\
            by default.
        covariance: compute covariance instead of correlation.
        fisher_z: apply the fisher small z transform to the correlations before\
            taking the gradient.
        undo_fisher_z: apply the inverse fisher small z transform to the input.
        presmooth_fwhm: smoothing kernel sizes are FWHM, not sigma.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCorrelationGradientOutputs`).
    """
    params = cifti_correlation_gradient_params(
        cifti=cifti,
        cifti_out=cifti_out,
        left_surface=left_surface,
        right_surface=right_surface,
        cerebellum_surface=cerebellum_surface,
        double_correlation=double_correlation,
        limit_gb=limit_gb,
        distance=distance,
        distance_=distance_,
        volume_kernel=volume_kernel,
        surface_kernel=surface_kernel,
        covariance=covariance,
        fisher_z=fisher_z,
        undo_fisher_z=undo_fisher_z,
        presmooth_fwhm=presmooth_fwhm,
    )
    return cifti_correlation_gradient_execute(params, runner)


__all__ = [
    "CIFTI_CORRELATION_GRADIENT_METADATA",
    "CiftiCorrelationGradientCerebellumSurfaceParamsDict",
    "CiftiCorrelationGradientCerebellumSurfaceParamsDictTagged",
    "CiftiCorrelationGradientDoubleCorrelationParamsDict",
    "CiftiCorrelationGradientDoubleCorrelationParamsDictTagged",
    "CiftiCorrelationGradientLeftSurfaceParamsDict",
    "CiftiCorrelationGradientLeftSurfaceParamsDictTagged",
    "CiftiCorrelationGradientOutputs",
    "CiftiCorrelationGradientParamsDict",
    "CiftiCorrelationGradientParamsDictTagged",
    "CiftiCorrelationGradientRightSurfaceParamsDict",
    "CiftiCorrelationGradientRightSurfaceParamsDictTagged",
    "cifti_correlation_gradient",
    "cifti_correlation_gradient_cerebellum_surface",
    "cifti_correlation_gradient_double_correlation",
    "cifti_correlation_gradient_execute",
    "cifti_correlation_gradient_left_surface",
    "cifti_correlation_gradient_params",
    "cifti_correlation_gradient_right_surface",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_AVERAGE_METADATA = Metadata(
    id="f30740343235dc708940e806f82a27a090de7bc3.workbench",
    name="cifti-average",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiAverageExcludeOutliersParamsDictNoTag = typing.TypedDict('_CiftiAverageExcludeOutliersParamsDictNoTag', {
    "sigma-below": float,
    "sigma-above": float,
})
CiftiAverageExcludeOutliersParamsDictTagged = typing.TypedDict('CiftiAverageExcludeOutliersParamsDictTagged', {
    "@type": typing.Literal["exclude-outliers"],
    "sigma-below": float,
    "sigma-above": float,
})
CiftiAverageExcludeOutliersParamsDict = _CiftiAverageExcludeOutliersParamsDictNoTag | CiftiAverageExcludeOutliersParamsDictTagged


_CiftiAverageCiftiParamsDictNoTag = typing.TypedDict('_CiftiAverageCiftiParamsDictNoTag', {
    "cifti-in": InputPathType,
    "weight": typing.NotRequired[float | None],
})
CiftiAverageCiftiParamsDictTagged = typing.TypedDict('CiftiAverageCiftiParamsDictTagged', {
    "@type": typing.Literal["cifti"],
    "cifti-in": InputPathType,
    "weight": typing.NotRequired[float | None],
})
CiftiAverageCiftiParamsDict = _CiftiAverageCiftiParamsDictNoTag | CiftiAverageCiftiParamsDictTagged


_CiftiAverageParamsDictNoTag = typing.TypedDict('_CiftiAverageParamsDictNoTag', {
    "cifti-out": str,
    "exclude-outliers": typing.NotRequired[CiftiAverageExcludeOutliersParamsDict | None],
    "cifti": typing.NotRequired[list[CiftiAverageCiftiParamsDict] | None],
    "limit-GB": typing.NotRequired[float | None],
})
CiftiAverageParamsDictTagged = typing.TypedDict('CiftiAverageParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-average"],
    "cifti-out": str,
    "exclude-outliers": typing.NotRequired[CiftiAverageExcludeOutliersParamsDict | None],
    "cifti": typing.NotRequired[list[CiftiAverageCiftiParamsDict] | None],
    "limit-GB": typing.NotRequired[float | None],
})
CiftiAverageParamsDict = _CiftiAverageParamsDictNoTag | CiftiAverageParamsDictTagged


def cifti_average_exclude_outliers(
    sigma_below: float,
    sigma_above: float,
) -> CiftiAverageExcludeOutliersParamsDictTagged:
    """
    Build parameters.
    
    Args:
        sigma_below: number of standard deviations below the mean to include.
        sigma_above: number of standard deviations above the mean to include.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "exclude-outliers",
        "sigma-below": sigma_below,
        "sigma-above": sigma_above,
    }
    return params


def cifti_average_exclude_outliers_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiAverageExcludeOutliersParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("sigma-below", None) is None:
        raise StyxValidationError("`sigma-below` must not be None")
    if not isinstance(params["sigma-below"], (float, int)):
        raise StyxValidationError(f'`sigma-below` has the wrong type: Received `{type(params.get("sigma-below", None))}` expected `float`')
    if params.get("sigma-above", None) is None:
        raise StyxValidationError("`sigma-above` must not be None")
    if not isinstance(params["sigma-above"], (float, int)):
        raise StyxValidationError(f'`sigma-above` has the wrong type: Received `{type(params.get("sigma-above", None))}` expected `float`')


def cifti_average_exclude_outliers_cargs(
    params: CiftiAverageExcludeOutliersParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-exclude-outliers",
        str(params.get("sigma-below", None)),
        str(params.get("sigma-above", None))
    ])
    return cargs


def cifti_average_cifti(
    cifti_in: InputPathType,
    weight: float | None = None,
) -> CiftiAverageCiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti file.
        weight: give a weight for this file\
            \
            the weight to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cifti",
        "cifti-in": cifti_in,
    }
    if weight is not None:
        params["weight"] = weight
    return params


def cifti_average_cifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiAverageCiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("weight", None) is not None:
        if not isinstance(params["weight"], (float, int)):
            raise StyxValidationError(f'`weight` has the wrong type: Received `{type(params.get("weight", None))}` expected `float | None`')


def cifti_average_cifti_cargs(
    params: CiftiAverageCiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cifti",
        execution.input_file(params.get("cifti-in", None))
    ])
    if params.get("weight", None) is not None:
        cargs.extend([
            "-weight",
            str(params.get("weight", None))
        ])
    return cargs


class CiftiAverageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiAverageParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_average_params(
    cifti_out: str,
    exclude_outliers: CiftiAverageExcludeOutliersParamsDict | None = None,
    cifti: list[CiftiAverageCiftiParamsDict] | None = None,
    limit_gb: float | None = None,
) -> CiftiAverageParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_out: output cifti file.
        exclude_outliers: exclude outliers by standard deviation of each\
            element across files.
        cifti: specify an input file.
        limit_gb: restrict memory used for file reading efficiency\
            \
            memory limit in gigabytes.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-average",
        "cifti-out": cifti_out,
    }
    if exclude_outliers is not None:
        params["exclude-outliers"] = exclude_outliers
    if cifti is not None:
        params["cifti"] = cifti
    if limit_gb is not None:
        params["limit-GB"] = limit_gb
    return params


def cifti_average_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiAverageParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("exclude-outliers", None) is not None:
        cifti_average_exclude_outliers_validate(params["exclude-outliers"])
    if params.get("cifti", None) is not None:
        if not isinstance(params["cifti"], list):
            raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `list[CiftiAverageCiftiParamsDict] | None`')
        for e in params["cifti"]:
            cifti_average_cifti_validate(e)
    if params.get("limit-GB", None) is not None:
        if not isinstance(params["limit-GB"], (float, int)):
            raise StyxValidationError(f'`limit-GB` has the wrong type: Received `{type(params.get("limit-GB", None))}` expected `float | None`')


def cifti_average_cargs(
    params: CiftiAverageParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-average"
    ])
    cargs.append(params.get("cifti-out", None))
    if params.get("exclude-outliers", None) is not None or params.get("cifti", None) is not None:
        cargs.extend([
            *(cifti_average_exclude_outliers_cargs(params.get("exclude-outliers", None), execution) if (params.get("exclude-outliers", None) is not None) else []),
            *([a for c in [cifti_average_cifti_cargs(s, execution) for s in params.get("cifti", None)] for a in c] if (params.get("cifti", None) is not None) else [])
        ])
    if params.get("limit-GB", None) is not None:
        cargs.extend([
            "-mem-limit",
            str(params.get("limit-GB", None))
        ])
    return cargs


def cifti_average_outputs(
    params: CiftiAverageParamsDict,
    execution: Execution,
) -> CiftiAverageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiAverageOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_average_execute(
    params: CiftiAverageParamsDict,
    runner: Runner | None = None,
) -> CiftiAverageOutputs:
    """
    AVERAGE CIFTI FILES.
    
    Averages cifti files together. Files without -weight specified are given a
    weight of 1. If -exclude-outliers is specified, at each element, the data
    across all files is taken as a set, its unweighted mean and sample standard
    deviation are found, and values outside the specified number of standard
    deviations are excluded from the (potentially weighted) average at that
    element.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiAverageOutputs`).
    """
    cifti_average_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_AVERAGE_METADATA)
    params = execution.params(params)
    cargs = cifti_average_cargs(params, execution)
    ret = cifti_average_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_average(
    cifti_out: str,
    exclude_outliers: CiftiAverageExcludeOutliersParamsDict | None = None,
    cifti: list[CiftiAverageCiftiParamsDict] | None = None,
    limit_gb: float | None = None,
    runner: Runner | None = None,
) -> CiftiAverageOutputs:
    """
    AVERAGE CIFTI FILES.
    
    Averages cifti files together. Files without -weight specified are given a
    weight of 1. If -exclude-outliers is specified, at each element, the data
    across all files is taken as a set, its unweighted mean and sample standard
    deviation are found, and values outside the specified number of standard
    deviations are excluded from the (potentially weighted) average at that
    element.
    
    Args:
        cifti_out: output cifti file.
        exclude_outliers: exclude outliers by standard deviation of each\
            element across files.
        cifti: specify an input file.
        limit_gb: restrict memory used for file reading efficiency\
            \
            memory limit in gigabytes.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiAverageOutputs`).
    """
    params = cifti_average_params(
        cifti_out=cifti_out,
        exclude_outliers=exclude_outliers,
        cifti=cifti,
        limit_gb=limit_gb,
    )
    return cifti_average_execute(params, runner)


__all__ = [
    "CIFTI_AVERAGE_METADATA",
    "CiftiAverageCiftiParamsDict",
    "CiftiAverageCiftiParamsDictTagged",
    "CiftiAverageExcludeOutliersParamsDict",
    "CiftiAverageExcludeOutliersParamsDictTagged",
    "CiftiAverageOutputs",
    "CiftiAverageParamsDict",
    "CiftiAverageParamsDictTagged",
    "cifti_average",
    "cifti_average_cifti",
    "cifti_average_exclude_outliers",
    "cifti_average_execute",
    "cifti_average_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_FILL_HOLES_METADATA = Metadata(
    id="447c0fe666083fe1acbc67cf224c9e527b125009.workbench",
    name="volume-fill-holes",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeFillHolesParamsDictNoTag = typing.TypedDict('_VolumeFillHolesParamsDictNoTag', {
    "volume-in": InputPathType,
    "volume-out": str,
})
VolumeFillHolesParamsDictTagged = typing.TypedDict('VolumeFillHolesParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-fill-holes"],
    "volume-in": InputPathType,
    "volume-out": str,
})
VolumeFillHolesParamsDict = _VolumeFillHolesParamsDictNoTag | VolumeFillHolesParamsDictTagged


class VolumeFillHolesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeFillHolesParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output ROI volume"""


def volume_fill_holes_params(
    volume_in: InputPathType,
    volume_out: str,
) -> VolumeFillHolesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input ROI volume.
        volume_out: the output ROI volume.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-fill-holes",
        "volume-in": volume_in,
        "volume-out": volume_out,
    }
    return params


def volume_fill_holes_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeFillHolesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')


def volume_fill_holes_cargs(
    params: VolumeFillHolesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-fill-holes"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(params.get("volume-out", None))
    return cargs


def volume_fill_holes_outputs(
    params: VolumeFillHolesParamsDict,
    execution: Execution,
) -> VolumeFillHolesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeFillHolesOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_fill_holes_execute(
    params: VolumeFillHolesParamsDict,
    runner: Runner | None = None,
) -> VolumeFillHolesOutputs:
    """
    FILL HOLES IN AN ROI VOLUME.
    
    Finds all face-connected parts that are not included in the ROI, and fills
    all but the largest one with ones.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeFillHolesOutputs`).
    """
    volume_fill_holes_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_FILL_HOLES_METADATA)
    params = execution.params(params)
    cargs = volume_fill_holes_cargs(params, execution)
    ret = volume_fill_holes_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_fill_holes(
    volume_in: InputPathType,
    volume_out: str,
    runner: Runner | None = None,
) -> VolumeFillHolesOutputs:
    """
    FILL HOLES IN AN ROI VOLUME.
    
    Finds all face-connected parts that are not included in the ROI, and fills
    all but the largest one with ones.
    
    Args:
        volume_in: the input ROI volume.
        volume_out: the output ROI volume.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeFillHolesOutputs`).
    """
    params = volume_fill_holes_params(
        volume_in=volume_in,
        volume_out=volume_out,
    )
    return volume_fill_holes_execute(params, runner)


__all__ = [
    "VOLUME_FILL_HOLES_METADATA",
    "VolumeFillHolesOutputs",
    "VolumeFillHolesParamsDict",
    "VolumeFillHolesParamsDictTagged",
    "volume_fill_holes",
    "volume_fill_holes_execute",
    "volume_fill_holes_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SPEC_FILE_RELOCATE_METADATA = Metadata(
    id="18d002a145082ef00fd24f099aca16f95fc0b29a.workbench",
    name="spec-file-relocate",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SpecFileRelocateParamsDictNoTag = typing.TypedDict('_SpecFileRelocateParamsDictNoTag', {
    "input-spec": str,
    "output-spec": str,
})
SpecFileRelocateParamsDictTagged = typing.TypedDict('SpecFileRelocateParamsDictTagged', {
    "@type": typing.Literal["workbench/spec-file-relocate"],
    "input-spec": str,
    "output-spec": str,
})
SpecFileRelocateParamsDict = _SpecFileRelocateParamsDictNoTag | SpecFileRelocateParamsDictTagged


class SpecFileRelocateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SpecFileRelocateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def spec_file_relocate_params(
    input_spec: str,
    output_spec: str,
) -> SpecFileRelocateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_spec: the spec file to use.
        output_spec: output - the new spec file to create.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/spec-file-relocate",
        "input-spec": input_spec,
        "output-spec": output_spec,
    }
    return params


def spec_file_relocate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SpecFileRelocateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input-spec", None) is None:
        raise StyxValidationError("`input-spec` must not be None")
    if not isinstance(params["input-spec"], str):
        raise StyxValidationError(f'`input-spec` has the wrong type: Received `{type(params.get("input-spec", None))}` expected `str`')
    if params.get("output-spec", None) is None:
        raise StyxValidationError("`output-spec` must not be None")
    if not isinstance(params["output-spec"], str):
        raise StyxValidationError(f'`output-spec` has the wrong type: Received `{type(params.get("output-spec", None))}` expected `str`')


def spec_file_relocate_cargs(
    params: SpecFileRelocateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-spec-file-relocate"
    ])
    cargs.append(params.get("input-spec", None))
    cargs.append(params.get("output-spec", None))
    return cargs


def spec_file_relocate_outputs(
    params: SpecFileRelocateParamsDict,
    execution: Execution,
) -> SpecFileRelocateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SpecFileRelocateOutputs(
        root=execution.output_file("."),
    )
    return ret


def spec_file_relocate_execute(
    params: SpecFileRelocateParamsDict,
    runner: Runner | None = None,
) -> SpecFileRelocateOutputs:
    """
    RECREATE SPEC FILE IN NEW LOCATION.
    
    Spec files contain internal relative paths, such that moving or copying a
    spec file will cause it to lose track of the files it refers to. This
    command makes a modified copy of the spec file, changing the relative paths
    to refer to the new relative locations of the files.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SpecFileRelocateOutputs`).
    """
    spec_file_relocate_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SPEC_FILE_RELOCATE_METADATA)
    params = execution.params(params)
    cargs = spec_file_relocate_cargs(params, execution)
    ret = spec_file_relocate_outputs(params, execution)
    execution.run(cargs)
    return ret


def spec_file_relocate(
    input_spec: str,
    output_spec: str,
    runner: Runner | None = None,
) -> SpecFileRelocateOutputs:
    """
    RECREATE SPEC FILE IN NEW LOCATION.
    
    Spec files contain internal relative paths, such that moving or copying a
    spec file will cause it to lose track of the files it refers to. This
    command makes a modified copy of the spec file, changing the relative paths
    to refer to the new relative locations of the files.
    
    Args:
        input_spec: the spec file to use.
        output_spec: output - the new spec file to create.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SpecFileRelocateOutputs`).
    """
    params = spec_file_relocate_params(
        input_spec=input_spec,
        output_spec=output_spec,
    )
    return spec_file_relocate_execute(params, runner)


__all__ = [
    "SPEC_FILE_RELOCATE_METADATA",
    "SpecFileRelocateOutputs",
    "SpecFileRelocateParamsDict",
    "SpecFileRelocateParamsDictTagged",
    "spec_file_relocate",
    "spec_file_relocate_execute",
    "spec_file_relocate_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

BORDER_LENGTH_METADATA = Metadata(
    id="cb4f6a9883ab1f8156dad010df4a97753b4a5a6d.workbench",
    name="border-length",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_BorderLengthParamsDictNoTag = typing.TypedDict('_BorderLengthParamsDictNoTag', {
    "border": InputPathType,
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
    "hide-border-name": bool,
    "separate-pieces": bool,
})
BorderLengthParamsDictTagged = typing.TypedDict('BorderLengthParamsDictTagged', {
    "@type": typing.Literal["workbench/border-length"],
    "border": InputPathType,
    "surface": InputPathType,
    "area-metric": typing.NotRequired[InputPathType | None],
    "hide-border-name": bool,
    "separate-pieces": bool,
})
BorderLengthParamsDict = _BorderLengthParamsDictNoTag | BorderLengthParamsDictTagged


class BorderLengthOutputs(typing.NamedTuple):
    """
    Output object returned when calling `BorderLengthParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def border_length_params(
    border: InputPathType,
    surface: InputPathType,
    area_metric: InputPathType | None = None,
    hide_border_name: bool = False,
    separate_pieces: bool = False,
) -> BorderLengthParamsDictTagged:
    """
    Build parameters.
    
    Args:
        border: the input border file.
        surface: the surface to measure the borders on.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        hide_border_name: don't print border name before each output.
        separate_pieces: report lengths for multi-part borders as separate\
            numbers.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/border-length",
        "border": border,
        "surface": surface,
        "hide-border-name": hide_border_name,
        "separate-pieces": separate_pieces,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    return params


def border_length_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `BorderLengthParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("border", None) is None:
        raise StyxValidationError("`border` must not be None")
    if not isinstance(params["border"], (pathlib.Path, str)):
        raise StyxValidationError(f'`border` has the wrong type: Received `{type(params.get("border", None))}` expected `InputPathType`')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("hide-border-name", False) is None:
        raise StyxValidationError("`hide-border-name` must not be None")
    if not isinstance(params["hide-border-name"], bool):
        raise StyxValidationError(f'`hide-border-name` has the wrong type: Received `{type(params.get("hide-border-name", False))}` expected `bool`')
    if params.get("separate-pieces", False) is None:
        raise StyxValidationError("`separate-pieces` must not be None")
    if not isinstance(params["separate-pieces"], bool):
        raise StyxValidationError(f'`separate-pieces` has the wrong type: Received `{type(params.get("separate-pieces", False))}` expected `bool`')


def border_length_cargs(
    params: BorderLengthParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-border-length"
    ])
    cargs.append(execution.input_file(params.get("border", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("hide-border-name", False):
        cargs.append("-hide-border-name")
    if params.get("separate-pieces", False):
        cargs.append("-separate-pieces")
    return cargs


def border_length_outputs(
    params: BorderLengthParamsDict,
    execution: Execution,
) -> BorderLengthOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = BorderLengthOutputs(
        root=execution.output_file("."),
    )
    return ret


def border_length_execute(
    params: BorderLengthParamsDict,
    runner: Runner | None = None,
) -> BorderLengthOutputs:
    """
    REPORT LENGTH OF BORDERS.
    
    For each border, print its length along the surface, in mm. If a border has
    multiple parts, their lengths are summed before printing, unless
    -separate-pieces is specified.
    
    The -corrected-areas option is intended for when the length is not
    meaningfully measurable on individual surfaces, it is only an approximate
    correction for the reduction in structure of a group average surface.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderLengthOutputs`).
    """
    border_length_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(BORDER_LENGTH_METADATA)
    params = execution.params(params)
    cargs = border_length_cargs(params, execution)
    ret = border_length_outputs(params, execution)
    execution.run(cargs)
    return ret


def border_length(
    border: InputPathType,
    surface: InputPathType,
    area_metric: InputPathType | None = None,
    hide_border_name: bool = False,
    separate_pieces: bool = False,
    runner: Runner | None = None,
) -> BorderLengthOutputs:
    """
    REPORT LENGTH OF BORDERS.
    
    For each border, print its length along the surface, in mm. If a border has
    multiple parts, their lengths are summed before printing, unless
    -separate-pieces is specified.
    
    The -corrected-areas option is intended for when the length is not
    meaningfully measurable on individual surfaces, it is only an approximate
    correction for the reduction in structure of a group average surface.
    
    Args:
        border: the input border file.
        surface: the surface to measure the borders on.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        hide_border_name: don't print border name before each output.
        separate_pieces: report lengths for multi-part borders as separate\
            numbers.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BorderLengthOutputs`).
    """
    params = border_length_params(
        border=border,
        surface=surface,
        area_metric=area_metric,
        hide_border_name=hide_border_name,
        separate_pieces=separate_pieces,
    )
    return border_length_execute(params, runner)


__all__ = [
    "BORDER_LENGTH_METADATA",
    "BorderLengthOutputs",
    "BorderLengthParamsDict",
    "BorderLengthParamsDictTagged",
    "border_length",
    "border_length_execute",
    "border_length_params",
]
